@@ -16,23 +16,27 @@
 //! Commands for console interaction.
 
 use crate::console::readline::read_line;
-use crate::console::{ClearType, Console, Position};
+use crate::console::{CharsXY, ClearType, Console, ConsoleClearable, Key};
 use async_trait::async_trait;
 use endbasic_core::ast::{ArgSep, Expr, Value, VarType};
 use endbasic_core::exec::Machine;
 use endbasic_core::syms::{
-    CallError, CallableMetadata, CallableMetadataBuilder, Command, CommandResult,
+    CallError, CallableMetadata, CallableMetadataBuilder, Command, CommandResult, Function,
+    FunctionResult, Symbols,
 };
 use std::cell::RefCell;
+use std::convert::TryFrom;
 use std::io;
 use std::rc::Rc;
 
 /// Category description for all symbols provided by this module.
 const CATEGORY: &str = "Console
-The EndBASIC console is the text-based display you are seeing: both the interpreter and the \
+The EndBASIC console is the display you are seeing: both the interpreter and the \
 effects of all commands happen within the same console.  There is no separate output window as \
-other didactical interpreters provide.
-The console is a matrix of variable size.  The upper left position is row 0 and column 0.  \
+other didactical interpreters provide.  This unified console supports text and, depending on the \
+output backend, graphics.  This help section focuses on the textual console; for information about \
+graphics, run HELP GRAPHICS.
+The text console is a matrix of variable size.  The upper left position is row 0 and column 0.  \
 Each position in this matrix contains a character and a color attribute.  The color attribute \
 indicates the foreground and background colors of that character.  There is a default attribute \
 to match the default settings of your terminal, which might not be a color: for example, in a \
@@ -41,9 +45,7 @@ respects the transparency whereas color 0 (black) does not.
 If you are writing a script and do not want the script to interfere with other parts of the \
 console, you should restrict the script to using only the INPUT and PRINT commands.
 Be aware that the console currently reacts poorly to size changes.  Avoid resizing your terminal \
-or web browser.  If you do resize them, however, restart the interpreter.
-Graphics are currently not supported but they are within the realm of possibility for a later \
-version.";
+or web browser.  If you do resize them, however, restart the interpreter.";
 
 /// The `CLS` command.
 pub struct ClsCommand {
@@ -123,9 +125,12 @@ impl Command for ColorCommand {
             }
         };
 
-        fn get_color(e: &Option<Expr>, machine: &mut Machine) -> Result<Option<u8>, CallError> {
+        async fn get_color(
+            e: &Option<Expr>,
+            machine: &mut Machine,
+        ) -> Result<Option<u8>, CallError> {
             match e {
-                Some(e) => match e.eval(machine.get_mut_symbols())? {
+                Some(e) => match e.eval(machine.get_mut_symbols()).await? {
                     Value::Integer(i) if i >= 0 && i <= std::u8::MAX as i32 => Ok(Some(i as u8)),
                     Value::Integer(_) => {
                         Err(CallError::ArgumentError("Color out of range".to_owned()))
@@ -136,14 +141,84 @@ impl Command for ColorCommand {
             }
         }
 
-        let fg = get_color(fg_expr, machine)?;
-        let bg = get_color(bg_expr, machine)?;
+        let fg = get_color(fg_expr, machine).await?;
+        let bg = get_color(bg_expr, machine).await?;
 
         self.console.borrow_mut().color(fg, bg)?;
         Ok(())
     }
 }
 
+/// The `INKEY` function.
+pub struct InKeyFunction {
+    metadata: CallableMetadata,
+    console: Rc<RefCell<dyn Console>>,
+}
+
+impl InKeyFunction {
+    /// Creates a new `INKEY` function that waits for a key press.
+    pub fn new(console: Rc<RefCell<dyn Console>>) -> Rc<Self> {
+        Rc::from(Self {
+            metadata: CallableMetadataBuilder::new("INKEY", VarType::Text)
+                .with_syntax("")
+                .with_category(CATEGORY)
+                .with_description(
+                    "Checks for an available key press and returns it.
+If a key press is available to be read, returns its name.  Otherwise, returns the empty string.  \
+The returned key matches its name, number, or symbol and maintains case.  In other words, \
+pressing the X key will return 'x' or 'X' depending on the SHIFT modifier.
+The following special keys are recognized: arrow keys (UP, DOWN, LEFT, RIGHT), backspace (BS), \
+end or CTRL+E (END), enter (ENTER), CTRL+D (EOF), escape (ESC), home or CTRL+A (HOME), \
+CTRL+C (INT), page up (PGUP), page down (PGDOWN), and tab (TAB).
+This function never blocks.  To wait for a key press, you need to explicitly poll the keyboard.  \
+For example, to wait until the escape key is pressed, you could do:
+k$ = \"\": WHILE k$ = <> \"ESC\": k = INKEY$(): SLEEP 0.01: WEND
+This non-blocking design lets you to combine the reception of multiple evens, such as from \
+GPIO_INPUT?, within the same loop.",
+                )
+                .build(),
+            console,
+        })
+    }
+}
+
+#[async_trait(?Send)]
+impl Function for InKeyFunction {
+    fn metadata(&self) -> &CallableMetadata {
+        &self.metadata
+    }
+
+    async fn exec(&self, args: &[Expr], _symbols: &mut Symbols) -> FunctionResult {
+        if !args.is_empty() {
+            return Err(CallError::ArgumentError("no arguments allowed".to_owned()));
+        }
+
+        let key = self.console.borrow_mut().poll_key().await?;
+        Ok(match key {
+            Some(Key::ArrowDown) => Value::Text("DOWN".to_owned()),
+            Some(Key::ArrowLeft) => Value::Text("LEFT".to_owned()),
+            Some(Key::ArrowRight) => Value::Text("RIGHT".to_owned()),
+            Some(Key::ArrowUp) => Value::Text("UP".to_owned()),
+
+            Some(Key::Backspace) => Value::Text("BS".to_owned()),
+            Some(Key::CarriageReturn) => Value::Text("ENTER".to_owned()),
+            Some(Key::Char(x)) => Value::Text(format!("{}", x)),
+            Some(Key::End) => Value::Text("END".to_owned()),
+            Some(Key::Eof) => Value::Text("EOF".to_owned()),
+            Some(Key::Escape) => Value::Text("ESC".to_owned()),
+            Some(Key::Home) => Value::Text("HOME".to_owned()),
+            Some(Key::Interrupt) => Value::Text("INT".to_owned()),
+            Some(Key::NewLine) => Value::Text("ENTER".to_owned()),
+            Some(Key::PageDown) => Value::Text("PGDOWN".to_owned()),
+            Some(Key::PageUp) => Value::Text("PGUP".to_owned()),
+            Some(Key::Tab) => Value::Text("TAB".to_owned()),
+            Some(Key::Unknown(_)) => Value::Text("".to_owned()),
+
+            None => Value::Text("".to_owned()),
+        })
+    }
+}
+
 /// The `INPUT` command.
 pub struct InputCommand {
     metadata: CallableMetadata,
@@ -183,7 +258,7 @@ impl Command for InputCommand {
         }
 
         let mut prompt = match &args[0].0 {
-            Some(e) => match e.eval(machine.get_mut_symbols())? {
+            Some(e) => match e.eval(machine.get_mut_symbols()).await? {
                 Value::Text(t) => t,
                 _ => {
                     return Err(CallError::ArgumentError(
@@ -241,7 +316,7 @@ impl LocateCommand {
     pub fn new(console: Rc<RefCell<dyn Console>>) -> Rc<Self> {
         Rc::from(Self {
             metadata: CallableMetadataBuilder::new("LOCATE", VarType::Void)
-                .with_syntax("row%, column%")
+                .with_syntax("column%, row%")
                 .with_category(CATEGORY)
                 .with_description("Moves the cursor to the given position.")
                 .build(),
@@ -260,43 +335,39 @@ impl Command for LocateCommand {
         if args.len() != 2 {
             return Err(CallError::ArgumentError("LOCATE takes two arguments".to_owned()));
         }
-        let (row_arg, column_arg) = (&args[0], &args[1]);
-        if row_arg.1 != ArgSep::Long {
+        let (column_arg, row_arg) = (&args[0], &args[1]);
+        if column_arg.1 != ArgSep::Long {
             return Err(CallError::ArgumentError(
                 "LOCATE expects arguments separated by a comma".to_owned(),
             ));
         }
-        debug_assert!(column_arg.1 == ArgSep::End);
-
-        let row = match &row_arg.0 {
-            Some(arg) => match arg.eval(machine.get_mut_symbols())? {
-                Value::Integer(i) => {
-                    if i < 0 {
-                        return Err(CallError::ArgumentError("Row cannot be negative".to_owned()));
-                    }
-                    i as usize
-                }
-                _ => return Err(CallError::ArgumentError("Row must be an integer".to_owned())),
-            },
-            None => return Err(CallError::ArgumentError("Row cannot be empty".to_owned())),
-        };
+        debug_assert!(row_arg.1 == ArgSep::End);
 
         let column = match &column_arg.0 {
-            Some(arg) => match arg.eval(machine.get_mut_symbols())? {
-                Value::Integer(i) => {
-                    if i < 0 {
-                        return Err(CallError::ArgumentError(
-                            "Column cannot be negative".to_owned(),
-                        ));
+            Some(arg) => match arg.eval(machine.get_mut_symbols()).await? {
+                Value::Integer(i) => match u16::try_from(i) {
+                    Ok(v) => v,
+                    Err(_) => {
+                        return Err(CallError::ArgumentError("Column out of range".to_owned()))
                     }
-                    i as usize
-                }
+                },
                 _ => return Err(CallError::ArgumentError("Column must be an integer".to_owned())),
             },
             None => return Err(CallError::ArgumentError("Column cannot be empty".to_owned())),
         };
 
-        self.console.borrow_mut().locate(Position { row, column })?;
+        let row = match &row_arg.0 {
+            Some(arg) => match arg.eval(machine.get_mut_symbols()).await? {
+                Value::Integer(i) => match u16::try_from(i) {
+                    Ok(v) => v,
+                    Err(_) => return Err(CallError::ArgumentError("Row out of range".to_owned())),
+                },
+                _ => return Err(CallError::ArgumentError("Row must be an integer".to_owned())),
+            },
+            None => return Err(CallError::ArgumentError("Row cannot be empty".to_owned())),
+        };
+
+        self.console.borrow_mut().locate(CharsXY::new(column, row))?;
         Ok(())
     }
 }
@@ -336,12 +407,17 @@ impl Command for PrintCommand {
         let mut text = String::new();
         for arg in args.iter() {
             if let Some(expr) = arg.0.as_ref() {
-                text += &expr.eval(machine.get_mut_symbols())?.to_string();
+                text += &expr.eval(machine.get_mut_symbols()).await?.to_string();
             }
             match arg.1 {
                 ArgSep::End => break,
                 ArgSep::Short => text += " ",
-                ArgSep::Long => text += "\t",
+                ArgSep::Long => {
+                    text += " ";
+                    while text.len() % 8 != 0 {
+                        text += " ";
+                    }
+                }
             }
         }
         self.console.borrow_mut().print(&text)?;
@@ -351,8 +427,10 @@ impl Command for PrintCommand {
 
 /// Adds all console-related commands for the given `console` to the `machine`.
 pub fn add_all(machine: &mut Machine, console: Rc<RefCell<dyn Console>>) {
+    machine.add_clearable(ConsoleClearable::new(console.clone()));
     machine.add_command(ClsCommand::new(console.clone()));
     machine.add_command(ColorCommand::new(console.clone()));
+    machine.add_function(InKeyFunction::new(console.clone()));
     machine.add_command(InputCommand::new(console.clone()));
     machine.add_command(LocateCommand::new(console.clone()));
     machine.add_command(PrintCommand::new(console));
@@ -399,6 +477,33 @@ mod tests {
         check_stmt_err("Color must be an integer", "COLOR 0, TRUE");
     }
 
+    #[test]
+    fn test_inkey_ok() {
+        Tester::default()
+            .run("result = INKEY()")
+            .expect_var("result", Value::Text("".to_owned()))
+            .check();
+
+        Tester::default()
+            .add_input_chars("x")
+            .run("result = INKEY()")
+            .expect_var("result", Value::Text("x".to_owned()))
+            .check();
+
+        Tester::default()
+            .add_input_keys(&[Key::CarriageReturn, Key::Backspace, Key::NewLine])
+            .run("r1 = INKEY(): r2 = INKEY(): r3 = INKEY()")
+            .expect_var("r1", Value::Text("ENTER".to_owned()))
+            .expect_var("r2", Value::Text("BS".to_owned()))
+            .expect_var("r3", Value::Text("ENTER".to_owned()))
+            .check();
+    }
+
+    #[test]
+    fn test_inkey_errors() {
+        check_expr_error("Syntax error in call to INKEY: no arguments allowed", "INKEY(1)");
+    }
+
     #[test]
     fn test_input_ok() {
         fn t<V: Into<Value>>(stmt: &str, input: &str, output: &str, var: &str, value: V) {
@@ -498,12 +603,12 @@ mod tests {
     fn test_locate_ok() {
         Tester::default()
             .run("LOCATE 0, 0")
-            .expect_output([CapturedOut::Locate(Position { row: 0, column: 0 })])
+            .expect_output([CapturedOut::Locate(CharsXY::default())])
             .check();
 
         Tester::default()
-            .run("LOCATE 1000, 2000")
-            .expect_output([CapturedOut::Locate(Position { row: 1000, column: 2000 })])
+            .run("LOCATE 63000, 64000")
+            .expect_output([CapturedOut::Locate(CharsXY::new(63000, 64000))])
             .check();
     }
 
@@ -514,21 +619,25 @@ mod tests {
         check_stmt_err("LOCATE takes two arguments", "LOCATE 1, 2, 3");
         check_stmt_err("LOCATE expects arguments separated by a comma", "LOCATE 1; 2");
 
-        check_stmt_err("Row cannot be negative", "LOCATE -1, 2");
-        check_stmt_err("Row must be an integer", "LOCATE TRUE, 2");
-        check_stmt_err("Row cannot be empty", "LOCATE , 2");
+        check_stmt_err("Column out of range", "LOCATE -1, 2");
+        check_stmt_err("Column out of range", "LOCATE 70000, 2");
+        check_stmt_err("Column must be an integer", "LOCATE TRUE, 2");
+        check_stmt_err("Column cannot be empty", "LOCATE , 2");
 
-        check_stmt_err("Column cannot be negative", "LOCATE 1, -2");
-        check_stmt_err("Column must be an integer", "LOCATE 1, TRUE");
-        check_stmt_err("Column cannot be empty", "LOCATE 1,");
+        check_stmt_err("Row out of range", "LOCATE 1, -2");
+        check_stmt_err("Row out of range", "LOCATE 1, 70000");
+        check_stmt_err("Row must be an integer", "LOCATE 1, TRUE");
+        check_stmt_err("Row cannot be empty", "LOCATE 1,");
     }
 
     #[test]
     fn test_print_ok() {
         Tester::default().run("PRINT").expect_prints([""]).check();
         Tester::default().run("PRINT ;").expect_prints([" "]).check();
-        Tester::default().run("PRINT ,").expect_prints(["\t"]).check();
-        Tester::default().run("PRINT ;,;,").expect_prints([" \t \t"]).check();
+        Tester::default().run("PRINT ,").expect_prints(["        "]).check();
+        Tester::default().run("PRINT ;,;,").expect_prints(["                "]).check();
+        Tester::default().run("PRINT \"abcdefg\", 1").expect_prints(["abcdefg 1"]).check();
+        Tester::default().run("PRINT \"abcdefgh\", 1").expect_prints(["abcdefgh        1"]).check();
 
         Tester::default().run("PRINT 3").expect_prints(["3"]).check();
         Tester::default().run("PRINT 3 = 5").expect_prints(["FALSE"]).check();
@@ -536,11 +645,11 @@ mod tests {
             .run("PRINT true;123;\"foo bar\"")
             .expect_prints(["TRUE 123 foo bar"])
             .check();
-        Tester::default().run("PRINT 6,1;3,5").expect_prints(["6\t1 3\t5"]).check();
+        Tester::default().run("PRINT 6,1;3,5").expect_prints(["6       1 3     5"]).check();
 
         Tester::default()
             .run(r#"word = "foo": PRINT word, word: PRINT word + "s""#)
-            .expect_prints(["foo\tfoo", "foos"])
+            .expect_prints(["foo     foo", "foos"])
             .expect_var("word", "foo")
             .check();
     }
