@@ -15,12 +15,30 @@
 
 //! Interactive line reader.
 
-use crate::console::{ClearType, Console, Key};
+use crate::console::{Console, Key};
 use std::io;
 
 /// Character to print when typing a secure string.
 const SECURE_CHAR: u8 = b'*';
 
+/// Refreshes the current input line to display `line` assuming that the cursor is currently
+/// offset by `pos` characters from the beginning of the input.
+fn update_line(console: &mut dyn Console, pos: usize, line: &str) -> io::Result<()> {
+    console.hide_cursor()?;
+    if pos > 0 {
+        console.move_within_line(-(pos as i16))?;
+    }
+    if !line.is_empty() {
+        console.write(line.as_bytes())?;
+    }
+    if line.len() < pos {
+        let diff = pos - line.len();
+        console.write(&vec![b' '; diff])?;
+        console.move_within_line(-(diff as i16))?;
+    }
+    console.show_cursor()
+}
+
 /// Reads a line of text interactively from the console, using the given `prompt` and pre-filling
 /// the input with `previous`.  If `history` is not `None`, then this appends the newly entered line
 /// into the history and allows navigating through it.
@@ -32,20 +50,20 @@ async fn read_line_interactive(
     echo: bool,
 ) -> io::Result<String> {
     let mut line = String::from(previous);
-    console.clear(ClearType::UntilNewLine)?;
     if !prompt.is_empty() || !line.is_empty() {
         if echo {
             console.write(format!("{}{}", prompt, line).as_bytes())?;
         } else {
             console.write(format!("{}{}", prompt, "*".repeat(line.len())).as_bytes())?;
         }
+        console.sync_now()?;
     }
 
     let width = {
         // Assumes that the prompt was printed at column 0.  If that was not the case, line length
         // calculation does not work.
         let console_size = console.size()?;
-        console_size.column - prompt.len()
+        usize::from(console_size.x) - prompt.len()
     };
 
     // Insertion position *within* the line, without accounting for the prompt.
@@ -72,15 +90,7 @@ async fn read_line_interactive(
                     history_pos -= 1;
                     line = history[history_pos].clone();
 
-                    console.hide_cursor()?;
-                    if pos > 0 {
-                        console.move_within_line(-(pos as i16))?;
-                        console.clear(ClearType::UntilNewLine)?;
-                    }
-                    if !line.is_empty() {
-                        console.write(line.as_bytes())?;
-                    }
-                    console.show_cursor()?;
+                    update_line(console, pos, &line)?;
 
                     pos = line.len();
                 }
@@ -96,15 +106,7 @@ async fn read_line_interactive(
                     history_pos += 1;
                     line = history[history_pos].clone();
 
-                    console.hide_cursor()?;
-                    if pos > 0 {
-                        console.move_within_line(-(pos as i16))?;
-                        console.clear(ClearType::UntilNewLine)?;
-                    }
-                    if !line.is_empty() {
-                        console.write(line.as_bytes())?;
-                    }
-                    console.show_cursor()?;
+                    update_line(console, pos, &line)?;
 
                     pos = line.len();
                 }
@@ -147,7 +149,7 @@ async fn read_line_interactive(
                 // instead of one.  Not sure if we should do this or if instead we should ensure
                 // the golden data we feed to the tests has single-character line endings.
                 if cfg!(not(target_os = "windows")) {
-                    console.write(&[b'\r', b'\n'])?;
+                    console.print("")?;
                     break;
                 }
             }
@@ -181,19 +183,42 @@ async fn read_line_interactive(
                 pos += 1;
             }
 
+            Key::End => {
+                let offset = line.len() - pos;
+                if offset > 0 {
+                    console.move_within_line(offset as i16)?;
+                    pos += offset;
+                }
+            }
+
             Key::Eof => return Err(io::Error::new(io::ErrorKind::UnexpectedEof, "EOF")),
 
             Key::Escape => {
                 // Intentionally ignored.
             }
 
+            Key::Home => {
+                if pos > 0 {
+                    console.move_within_line(-(pos as i16))?;
+                    pos = 0;
+                }
+            }
+
             Key::Interrupt => return Err(io::Error::new(io::ErrorKind::Interrupted, "Ctrl+C")),
 
             Key::NewLine => {
-                console.write(&[b'\r', b'\n'])?;
+                console.print("")?;
                 break;
             }
 
+            Key::PageDown | Key::PageUp => {
+                // Intentionally ignored.
+            }
+
+            Key::Tab => {
+                // TODO(jmmv): Would be nice to have some form of auto-completion.
+            }
+
             // TODO(jmmv): Should do something smarter with unknown keys.
             Key::Unknown(_) => (),
         }
@@ -231,10 +256,13 @@ async fn read_line_raw(console: &mut dyn Console) -> io::Result<String> {
                 }
             }
             Key::Char(ch) => line.push(ch),
+            Key::End | Key::Home => (),
             Key::Escape => (),
             Key::Eof => return Err(io::Error::new(io::ErrorKind::UnexpectedEof, "EOF")),
             Key::Interrupt => return Err(io::Error::new(io::ErrorKind::Interrupted, "Ctrl+C")),
             Key::NewLine => break,
+            Key::PageDown | Key::PageUp => (),
+            Key::Tab => (),
             Key::Unknown(bad_input) => line += &bad_input,
         }
     }
@@ -276,7 +304,7 @@ pub(crate) async fn read_line_secure(
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::console::Position;
+    use crate::console::CharsXY;
     use crate::testutils::*;
     use futures_lite::future::block_on;
 
@@ -304,7 +332,7 @@ mod tests {
                 history: None,
                 echo: true,
                 exp_line: "",
-                exp_output: vec![CapturedOut::Clear(ClearType::UntilNewLine)],
+                exp_output: vec![],
                 exp_history: None,
             }
         }
@@ -379,11 +407,11 @@ mod tests {
         /// executes the test.
         fn accept(mut self) {
             self.keys.push(Key::NewLine);
-            self.exp_output.push(CapturedOut::Write(vec![b'\r', b'\n']));
+            self.exp_output.push(CapturedOut::Print("".to_owned()));
 
             let mut console = MockConsole::default();
             console.add_input_keys(&self.keys);
-            console.set_size(Position { row: 5, column: 15 });
+            console.set_size(CharsXY::new(15, 5));
             let line = match self.history.as_mut() {
                 Some(mut history) => block_on(read_line_interactive(
                     &mut console,
@@ -421,6 +449,7 @@ mod tests {
         ReadLineInteractiveTest::default()
             .set_prompt("Ready> ")
             .add_output(CapturedOut::Write(b"Ready> ".to_vec()))
+            .add_output(CapturedOut::SyncNow)
             // -
             .add_key_chars("hello")
             .add_output_bytes(b"hello")
@@ -431,6 +460,7 @@ mod tests {
         ReadLineInteractiveTest::default()
             .set_prompt("Cannot delete")
             .add_output(CapturedOut::Write(b"Cannot delete".to_vec()))
+            .add_output(CapturedOut::SyncNow)
             // -
             .add_key(Key::Backspace)
             .accept();
@@ -448,6 +478,7 @@ mod tests {
         ReadLineInteractiveTest::default()
             .set_previous("123")
             .add_output(CapturedOut::Write(b"123".to_vec()))
+            .add_output(CapturedOut::SyncNow)
             // -
             .add_key_chars("hello")
             .add_output_bytes(b"hello")
@@ -549,6 +580,7 @@ mod tests {
         ReadLineInteractiveTest::default()
             .set_previous("12")
             .add_output(CapturedOut::Write(b"12".to_vec()))
+            .add_output(CapturedOut::SyncNow)
             // -
             .add_key(Key::ArrowLeft)
             .add_output(CapturedOut::MoveWithinLine(-1))
@@ -577,6 +609,60 @@ mod tests {
             .accept();
     }
 
+    #[test]
+    fn test_read_line_interactive_test_home_end() {
+        ReadLineInteractiveTest::default()
+            .set_previous("sample text")
+            .add_output(CapturedOut::Write(b"sample text".to_vec()))
+            .add_output(CapturedOut::SyncNow)
+            // -
+            .add_key(Key::End)
+            // -
+            .add_key(Key::Home)
+            .add_output(CapturedOut::MoveWithinLine(-11))
+            // -
+            .add_key(Key::Home)
+            // -
+            .add_key(Key::Char('>'))
+            .add_output(CapturedOut::HideCursor)
+            .add_output_bytes(b">")
+            .add_output(CapturedOut::Write(b"sample text".to_vec()))
+            .add_output(CapturedOut::MoveWithinLine(-11))
+            .add_output(CapturedOut::ShowCursor)
+            // -
+            .add_key(Key::End)
+            .add_output(CapturedOut::MoveWithinLine(11))
+            // -
+            .add_key(Key::Char('<'))
+            .add_output_bytes(b"<")
+            // -
+            .add_key(Key::ArrowLeft)
+            .add_output(CapturedOut::MoveWithinLine(-1))
+            // -
+            .add_key(Key::ArrowLeft)
+            .add_output(CapturedOut::MoveWithinLine(-1))
+            // -
+            .add_key(Key::ArrowLeft)
+            .add_output(CapturedOut::MoveWithinLine(-1))
+            // -
+            .add_key(Key::ArrowLeft)
+            .add_output(CapturedOut::MoveWithinLine(-1))
+            // -
+            .add_key(Key::ArrowLeft)
+            .add_output(CapturedOut::MoveWithinLine(-1))
+            // -
+            .add_key(Key::Backspace)
+            .add_output(CapturedOut::HideCursor)
+            .add_output(CapturedOut::MoveWithinLine(-1))
+            .add_output(CapturedOut::Write(b"text<".to_vec()))
+            .add_output_bytes(b" ")
+            .add_output(CapturedOut::MoveWithinLine(-6))
+            .add_output(CapturedOut::ShowCursor)
+            // -
+            .set_line(">sampletext<")
+            .accept();
+    }
+
     #[test]
     fn test_read_line_interactive_horizontal_scrolling_not_implemented() {
         ReadLineInteractiveTest::default()
@@ -605,6 +691,7 @@ mod tests {
             .set_prompt("12345")
             .set_previous("67890")
             .add_output(CapturedOut::Write(b"1234567890".to_vec()))
+            .add_output(CapturedOut::SyncNow)
             // -
             .add_key_chars("1234567890")
             .add_output_bytes(b"1234")
@@ -665,6 +752,7 @@ mod tests {
         ReadLineInteractiveTest::default()
             .set_prompt("? ")
             .add_output(CapturedOut::Write(b"? ".to_vec()))
+            .add_output(CapturedOut::SyncNow)
             //
             .set_history(
                 vec!["first".to_owned(), "second".to_owned(), "third".to_owned()],
@@ -679,15 +767,15 @@ mod tests {
             .add_key(Key::ArrowUp)
             .add_output(CapturedOut::HideCursor)
             .add_output(CapturedOut::MoveWithinLine(-5))
-            .add_output(CapturedOut::Clear(ClearType::UntilNewLine))
             .add_output(CapturedOut::Write(b"second".to_vec()))
             .add_output(CapturedOut::ShowCursor)
             //
             .add_key(Key::ArrowUp)
             .add_output(CapturedOut::HideCursor)
             .add_output(CapturedOut::MoveWithinLine(-6))
-            .add_output(CapturedOut::Clear(ClearType::UntilNewLine))
             .add_output(CapturedOut::Write(b"first".to_vec()))
+            .add_output(CapturedOut::Write(b" ".to_vec()))
+            .add_output(CapturedOut::MoveWithinLine(-1))
             .add_output(CapturedOut::ShowCursor)
             //
             .add_key(Key::ArrowUp)
@@ -695,21 +783,22 @@ mod tests {
             .add_key(Key::ArrowDown)
             .add_output(CapturedOut::HideCursor)
             .add_output(CapturedOut::MoveWithinLine(-5))
-            .add_output(CapturedOut::Clear(ClearType::UntilNewLine))
             .add_output(CapturedOut::Write(b"second".to_vec()))
             .add_output(CapturedOut::ShowCursor)
             //
             .add_key(Key::ArrowDown)
             .add_output(CapturedOut::HideCursor)
             .add_output(CapturedOut::MoveWithinLine(-6))
-            .add_output(CapturedOut::Clear(ClearType::UntilNewLine))
             .add_output(CapturedOut::Write(b"third".to_vec()))
+            .add_output(CapturedOut::Write(b" ".to_vec()))
+            .add_output(CapturedOut::MoveWithinLine(-1))
             .add_output(CapturedOut::ShowCursor)
             //
             .add_key(Key::ArrowDown)
             .add_output(CapturedOut::HideCursor)
             .add_output(CapturedOut::MoveWithinLine(-5))
-            .add_output(CapturedOut::Clear(ClearType::UntilNewLine))
+            .add_output(CapturedOut::Write(b"     ".to_vec()))
+            .add_output(CapturedOut::MoveWithinLine(-5))
             .add_output(CapturedOut::ShowCursor)
             //
             .add_key(Key::ArrowDown)
@@ -722,6 +811,7 @@ mod tests {
         ReadLineInteractiveTest::default()
             .set_prompt("? ")
             .add_output(CapturedOut::Write(b"? ".to_vec()))
+            .add_output(CapturedOut::SyncNow)
             //
             .set_history(
                 vec!["first".to_owned(), "second".to_owned(), "third".to_owned()],
@@ -741,7 +831,6 @@ mod tests {
             .add_key(Key::ArrowUp)
             .add_output(CapturedOut::HideCursor)
             .add_output(CapturedOut::MoveWithinLine(-5))
-            .add_output(CapturedOut::Clear(ClearType::UntilNewLine))
             .add_output(CapturedOut::Write(b"second".to_vec()))
             .add_output(CapturedOut::ShowCursor)
             //
@@ -772,6 +861,9 @@ mod tests {
             .add_output_bytes(b"not ")
             // -
             .add_key(Key::Escape)
+            .add_key(Key::PageDown)
+            .add_key(Key::PageUp)
+            .add_key(Key::Tab)
             // -
             .add_key_chars("affected")
             .add_output_bytes(b"affected")
@@ -787,6 +879,7 @@ mod tests {
             .set_prompt("> ")
             .set_previous("pass1234")
             .add_output(CapturedOut::Write(b"> ********".to_vec()))
+            .add_output(CapturedOut::SyncNow)
             // -
             .add_key_chars("56")
             .add_output_bytes(b"**")
@@ -820,16 +913,16 @@ mod tests {
         let mut console = MockConsole::default();
         console.set_interactive(true);
         console.add_input_keys(&[Key::Char('1'), Key::Char('5'), Key::NewLine]);
-        console.set_size(Position { row: 5, column: 15 });
+        console.set_size(CharsXY::new(15, 5));
         let line = block_on(read_line_secure(&mut console, "> ")).unwrap();
         assert_eq!("15", &line);
         assert_eq!(
             &[
-                CapturedOut::Clear(ClearType::UntilNewLine),
                 CapturedOut::Write(vec![b'>', b' ']),
+                CapturedOut::SyncNow,
                 CapturedOut::Write(vec![b'*']),
                 CapturedOut::Write(vec![b'*']),
-                CapturedOut::Write(vec![b'\r', b'\n']),
+                CapturedOut::Print("".to_owned()),
             ],
             console.captured_out()
         );
