@@ -134,7 +134,7 @@ account is ready.",
 
     /// Performs the login workflow against the server.
     async fn do_login(&self, username: &str, password: &str) -> CommandResult {
-        let access_token = self.service.borrow_mut().authenticate(&username, &password).await?;
+        let access_token = self.service.borrow_mut().authenticate(username, password).await?;
 
         let mut first = true;
         let mut data = HashMap::new();
@@ -198,20 +198,22 @@ impl Command for LoginCommand {
         }
 
         let (username, password) = match args {
-            [(Some(username), ArgSep::End)] => match username.eval(machine.get_mut_symbols())? {
-                Value::Text(username) => {
-                    let password =
-                        read_line_secure(&mut *self.console.borrow_mut(), "Password: ").await?;
-                    (username, password)
+            [(Some(username), ArgSep::End)] => {
+                match username.eval(machine.get_mut_symbols()).await? {
+                    Value::Text(username) => {
+                        let password =
+                            read_line_secure(&mut *self.console.borrow_mut(), "Password: ").await?;
+                        (username, password)
+                    }
+                    _ => {
+                        return Err(CallError::ArgumentError(
+                            "LOGIN requires a string as the username".to_owned(),
+                        ))
+                    }
                 }
-                _ => {
-                    return Err(CallError::ArgumentError(
-                        "LOGIN requires a string as the username".to_owned(),
-                    ))
-                }
-            },
+            }
             [(Some(username), ArgSep::Long), (Some(password), ArgSep::End)] => {
-                let username = match username.eval(machine.get_mut_symbols())? {
+                let username = match username.eval(machine.get_mut_symbols()).await? {
                     Value::Text(username) => username,
                     _ => {
                         return Err(CallError::ArgumentError(
@@ -219,7 +221,7 @@ impl Command for LoginCommand {
                         ))
                     }
                 };
-                let password = match password.eval(machine.get_mut_symbols())? {
+                let password = match password.eval(machine.get_mut_symbols()).await? {
                     Value::Text(password) => password,
                     _ => {
                         return Err(CallError::ArgumentError(
@@ -331,7 +333,7 @@ impl Command for ShareCommand {
         }
 
         let filename = match &args[0].0 {
-            Some(e) => match e.eval(machine.get_mut_symbols())? {
+            Some(e) => match e.eval(machine.get_mut_symbols()).await? {
                 Value::Text(t) => t,
                 _ => {
                     return Err(CallError::ArgumentError(
@@ -367,7 +369,7 @@ impl Command for ShareCommand {
                         "SHARE requires arguments to be separated by commas".to_owned(),
                     ))
                 }
-                (Some(acl), _) => match acl.eval(machine.get_mut_symbols())? {
+                (Some(acl), _) => match acl.eval(machine.get_mut_symbols()).await? {
                     Value::Text(t) => ShareCommand::parse_acl(t, &mut add, &mut remove)?,
                     _ => {
                         return Err(CallError::ArgumentError(
@@ -397,7 +399,6 @@ pub(crate) fn add_all(
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::console::ClearType;
     use crate::testutils::*;
 
     #[test]
@@ -423,14 +424,11 @@ mod tests {
         assert!(!storage.borrow().mounted().contains_key("CLOUD"));
 
         t.get_console().borrow_mut().set_interactive(true);
-        let mut exp_output = vec![
-            CapturedOut::Clear(ClearType::UntilNewLine),
-            CapturedOut::Write(b"Password: ".to_vec()),
-        ];
+        let mut exp_output = vec![CapturedOut::Write(b"Password: ".to_vec()), CapturedOut::SyncNow];
         for _ in 0..MockService::PASSWORD.len() {
             exp_output.push(CapturedOut::Write(vec![b'*']));
         }
-        exp_output.push(CapturedOut::Write(vec![b'\r', b'\n']));
+        exp_output.push(CapturedOut::Print("".to_owned()));
 
         t.add_input_chars(MockService::PASSWORD)
             .add_input_chars("\n")
